@@ -11,7 +11,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#   include <config.h>
+#include <config.h>
 #endif
 
 #include <string.h>
@@ -28,7 +28,7 @@
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
-#include <libxfce4panel/xfce-panel-plugin.h>
+#include <libxfce4panel/libxfce4panel.h>
 #include "parsetree.h"
 #include "parser.h"
 #include "eval.h"
@@ -38,6 +38,7 @@
 #define DEFAULT_DEGREES FALSE
 #define DEFAULT_SIZE 20
 #define DEFAULT_HIST_SIZE 25
+#define DEFAULT_MOVE_CURSOR FALSE
 #define DEFAULT_OUTPUT_BASE 10
 
 
@@ -57,12 +58,13 @@ typedef struct {
     gboolean degrees; // Degrees or radians for trigonometric functions?
     gint size;		  // Size of comboboxentry 
     gint hist_size;
+    gboolean move_cursor;
     gint output_base; // 10 = decimal, 16 = hexadecimal. Other values are not supported
 } CalcPlugin;
 
 
 
-void calc_save_config(XfcePanelPlugin *plugin, CalcPlugin *calc)
+static void calc_save_config(XfcePanelPlugin *plugin, CalcPlugin *calc)
 {
     XfceRc *rc;
     gchar *file;
@@ -77,6 +79,7 @@ void calc_save_config(XfcePanelPlugin *plugin, CalcPlugin *calc)
         xfce_rc_write_bool_entry(rc, "degrees", calc->degrees);
         xfce_rc_write_int_entry(rc, "size", calc->size);
         xfce_rc_write_int_entry(rc, "hist_size", calc->hist_size);
+        xfce_rc_write_int_entry(rc, "move_cursor", calc->move_cursor);
         xfce_rc_write_int_entry(rc, "output_base", calc->output_base);
         xfce_rc_close(rc);
     }
@@ -100,6 +103,7 @@ static void calc_read_config(CalcPlugin *calc)
         calc->degrees = xfce_rc_read_bool_entry(rc, "degrees", DEFAULT_DEGREES);
         calc->size = xfce_rc_read_int_entry(rc, "size", DEFAULT_SIZE);
         calc->hist_size = xfce_rc_read_int_entry(rc, "hist_size", DEFAULT_HIST_SIZE);
+        calc->move_cursor = xfce_rc_read_int_entry(rc, "move_cursor", DEFAULT_MOVE_CURSOR);
         calc->output_base = xfce_rc_read_int_entry(rc, "output_base", DEFAULT_OUTPUT_BASE);
         xfce_rc_close(rc);
     } else {
@@ -107,6 +111,7 @@ static void calc_read_config(CalcPlugin *calc)
         calc->degrees = DEFAULT_DEGREES;
         calc->size = DEFAULT_SIZE;
         calc->hist_size = DEFAULT_HIST_SIZE;
+        calc->move_cursor = DEFAULT_MOVE_CURSOR;
         calc->output_base = DEFAULT_OUTPUT_BASE;
     }
 }
@@ -126,7 +131,7 @@ static GList *add_to_expr_hist(GList *ehist, gint hist_size, const gchar *str)
     ehist = g_list_append(ehist, g_strdup(str));
 
     // Remove oldest, if list is growing too long.
-    if (g_list_length(ehist) > hist_size) {
+    if (g_list_length(ehist) > (guint)hist_size) {
         elem = g_list_first(ehist);
         g_free(elem->data);
         ehist = g_list_delete_link(ehist, elem);
@@ -183,7 +188,8 @@ static void entry_enter_cb(GtkEntry *entry, CalcPlugin *calc)
             output = g_strdup_printf("%.16g", r);
         }
         gtk_entry_set_text(entry, output);
-        gtk_editable_set_position(GTK_EDITABLE(entry), -1);
+        if(calc->move_cursor)
+            gtk_editable_set_position(GTK_EDITABLE(entry), -1);
         g_free(output);
         free_parsetree(parsetree);
     }
@@ -211,7 +217,7 @@ static CalcPlugin *calc_new(XfcePanelPlugin *plugin)
     GtkOrientation orientation;
     GtkWidget *icon, *combo, *entry;
 
-    calc = panel_slice_new0(CalcPlugin);
+    calc = g_slice_new0(CalcPlugin);
     calc->plugin = plugin;
     calc_read_config(calc);
 
@@ -243,6 +249,9 @@ static CalcPlugin *calc_new(XfcePanelPlugin *plugin)
 
     gtk_entry_set_max_length(GTK_ENTRY(entry), 50);
     gtk_entry_set_width_chars(GTK_ENTRY(entry), calc->size);
+#if GTK_CHECK_VERSION (3, 22, 20)
+    gtk_entry_set_input_hints (GTK_ENTRY(entry), GTK_INPUT_HINT_NO_EMOJI);
+#endif
 
     return calc;
 }
@@ -276,7 +285,7 @@ static void calc_free(XfcePanelPlugin *plugin, CalcPlugin *calc)
      * freeing of expr_hist enough?
      */
 
-    panel_slice_free(CalcPlugin, calc);
+    g_slice_free(CalcPlugin, calc);
 }
 
 
@@ -307,12 +316,14 @@ static gboolean calc_size_changed(XfcePanelPlugin *plugin, gint size,
 static gboolean calc_plugin_update_size(XfcePanelPlugin *plugin, gint size,
                                         CalcPlugin *calc)
 {
+	GtkWidget *entry;
+
 	g_assert(calc);
 	g_assert(calc->combo);
 
 	calc->size = size;
 
-	GtkWidget *entry = gtk_bin_get_child (GTK_BIN (calc->combo));
+	entry = gtk_bin_get_child (GTK_BIN (calc->combo));
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), size);
 
 	return TRUE;
@@ -333,6 +344,13 @@ static void calc_hist_size_changed(GtkSpinButton *spin, CalcPlugin *calc)
 }
 
 
+static void calc_move_cursor_changed(GtkToggleButton *button, CalcPlugin *calc)
+{
+    g_assert(calc);
+    calc->move_cursor = !gtk_toggle_button_get_active(button);
+}
+
+
 /* Called when the "trigonometrics use degree/radians" menu items change state.
 
    Note that since they are radio buttons, grouped together, they will allways
@@ -355,6 +373,8 @@ static void angle_unit_chosen(GtkCheckMenuItem *button, CalcPlugin *calc)
 
 static void hexadecimal_output_chosen(GtkCheckMenuItem *button, CalcPlugin *calc)
 {
+    GtkWidget *entry;
+
     g_assert(button == (GtkCheckMenuItem *)calc->hexadecimal_button);
 
     if (gtk_check_menu_item_get_active(button))
@@ -363,7 +383,7 @@ static void hexadecimal_output_chosen(GtkCheckMenuItem *button, CalcPlugin *calc
         calc->output_base = 10;
     }
     // convert current value to new base
-    GtkWidget *entry = gtk_bin_get_child (GTK_BIN (calc->combo));
+    entry = gtk_bin_get_child (GTK_BIN (calc->combo));
     entry_enter_cb(GTK_ENTRY(entry), calc);
 }
 
@@ -371,7 +391,7 @@ static void hexadecimal_output_chosen(GtkCheckMenuItem *button, CalcPlugin *calc
 static void calc_dialog_response(GtkWidget *dialog, gint response,
                                  CalcPlugin *calc)
 {
-    if (response == GTK_RESPONSE_OK) {
+    if (response == GTK_RESPONSE_OK || response == GTK_RESPONSE_DELETE_EVENT) {
         g_object_set_data(G_OBJECT(calc->plugin), "dialog", NULL);
         xfce_panel_plugin_unblock_menu(calc->plugin);
         calc_save_config(calc->plugin, calc);
@@ -391,6 +411,8 @@ static void calc_configure(XfcePanelPlugin *plugin, CalcPlugin *calc)
     GtkWidget *hbox;
     GtkWidget *size_label;
     GtkWidget *size_spin;
+    GtkWidget *button;
+    GtkWidget *image;
     GtkAdjustment *adjustment;
 
     xfce_panel_plugin_block_menu(plugin);
@@ -401,6 +423,10 @@ static void calc_configure(XfcePanelPlugin *plugin, CalcPlugin *calc)
                        GTK_DIALOG_DESTROY_WITH_PARENT,
                        _("_Close"), GTK_RESPONSE_OK, NULL);
 
+#if LIBXFCE4UI_CHECK_VERSION (4, 15, 1)
+  xfce_titled_dialog_create_action_area (XFCE_TITLED_DIALOG (dialog));
+#endif
+
     gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
     gtk_window_set_icon_name(GTK_WINDOW(dialog), "xfce4-calculator-plugin");
 
@@ -413,6 +439,7 @@ static void calc_configure(XfcePanelPlugin *plugin, CalcPlugin *calc)
 
 	area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
+    /* Appearance */
 	frame = xfce_gtk_frame_box_new (_("Appearance"), &bin);
 
 	gtk_container_set_border_width(GTK_CONTAINER(frame), 6);
@@ -429,12 +456,13 @@ static void calc_configure(XfcePanelPlugin *plugin, CalcPlugin *calc)
 	adjustment = gtk_adjustment_new(calc->size, 5, 100, 1, 5, 10);
 	size_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
 	gtk_widget_add_mnemonic_label(size_spin, size_label);
-	gtk_box_pack_start(GTK_BOX(hbox), size_spin, FALSE, TRUE, 0);
+	gtk_box_pack_end(GTK_BOX(hbox), size_spin, FALSE, TRUE, 0);
 	gtk_widget_show(size_spin);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(size_spin), calc->size);
 	g_signal_connect(size_spin, "value-changed",
                      G_CALLBACK(calc_plugin_size_changed), calc);
 
+    /* History */
     frame = xfce_gtk_frame_box_new (_("History"), &bin);
 
     gtk_container_set_border_width(GTK_CONTAINER (frame), 6);
@@ -450,35 +478,54 @@ static void calc_configure(XfcePanelPlugin *plugin, CalcPlugin *calc)
     gtk_widget_show(size_label);
     adjustment = gtk_adjustment_new(calc->hist_size, 0, 100, 1, 10, 20);
     size_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), size_spin, FALSE, TRUE, 0);
+    gtk_box_pack_end(GTK_BOX(hbox), size_spin, FALSE, TRUE, 0);
     gtk_widget_show (size_spin);
     g_signal_connect(size_spin, "value-changed",
                      G_CALLBACK(calc_hist_size_changed), calc);
 
+    /* Behavior */
+    frame = xfce_gtk_frame_box_new (_("Behavior"), &bin);
+
+    gtk_container_set_border_width(GTK_CONTAINER (frame), 6);
+    gtk_box_pack_start(GTK_BOX(area), frame, TRUE, TRUE, 0);
+    gtk_widget_show(frame);
+
+    button = gtk_check_button_new_with_label(_("Do not move cursor after calculation"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), !calc->move_cursor);
+    gtk_container_add(GTK_CONTAINER(bin), button);
+    gtk_widget_show(button);
+    g_signal_connect(button, "toggled",
+                     G_CALLBACK(calc_move_cursor_changed), calc);
+
+    /* add the "Close" button */
+    button = gtk_button_new_with_mnemonic (_("_Close"));
+    image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_image (GTK_BUTTON (button), image);
+#if LIBXFCE4UI_CHECK_VERSION (4, 15, 1)
+    xfce_titled_dialog_add_action_widget (XFCE_TITLED_DIALOG (dialog), button, GTK_RESPONSE_OK);
+#else
+    gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
+#endif
+    gtk_widget_show (button);
+
     gtk_widget_show(dialog);
 }
 
-void calc_about (XfcePanelPlugin *plugin)
+static void calc_about (XfcePanelPlugin *plugin)
 {
-       GdkPixbuf *icon;
    const gchar *authors[] = {
       "Erik Edelmann <erik.edelmann@iki.fi>",
       "Adrian Dimitrov <enzo_01@abv.bg>",
       "Roland Kuebert <roland@upic.de>", NULL };
-   icon = xfce_panel_pixbuf_from_source("xfce4-calculator-plugin", NULL, 32);
    gtk_show_about_dialog(NULL,
-      "logo", icon,
+      "logo-icon-name", "xfce4-calculator-plugin",
       "license", xfce_get_license_text (XFCE_LICENSE_TEXT_GPL),
       "version", PACKAGE_VERSION,
       "program-name", PACKAGE_NAME,
       "comments", _("Calculator for Xfce panel"),
-      "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-calculator-plugin",
-      "copyright", _("Copyright (c) 2003-2019\n"),
+      "website", "https://docs.xfce.org/panel-plugins/xfce4-calculator-plugin",
+      "copyright", "Copyright \302\251 2003-2023 The Xfce development team",
       "authors", authors, NULL);
-
-   if(icon)
-      g_object_unref(G_OBJECT(icon));
-
 }
 
 
