@@ -9,13 +9,17 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
  *  USA.
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -24,7 +28,7 @@
 
 static gboolean isoperator(int c)
 {
-    if (c == '+' || c == '-' || c == '*' || c == '/' || c == '^')
+    if (c == '+' || c == '-' || c == '*' || c == '/' || c == '%' || c == '^')
         return TRUE;
     else
         return FALSE;
@@ -39,11 +43,11 @@ static token_t *get_next_token(const char *input, int *index)
 {
     const char *t;
     token_t *token;
-    int i;
+    int i, n = 0;
 
     g_assert(input);
     g_assert(index);
-    g_assert(*index <= strlen(input));
+    g_assert((size_t)(*index) <= strlen(input));
 
     i = *index;
 
@@ -77,7 +81,6 @@ static token_t *get_next_token(const char *input, int *index)
     } else if (isalpha(input[i])) {
         token->type = TOK_IDENTIFIER;
 
-        int n = 0;
         while (isalnum(input[i]) && n < MAX_ID_LEN) {
             token->val.id[n] = input[i];
             n++, i++;
