@@ -9,13 +9,17 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
  *  USA.
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
@@ -121,7 +125,7 @@ static gboolean is_mult_op(char op)
     /*
     return op[0] == '/' || (op[0] == '*' && op[1] == '\0')
     */
-    return op == '/' || op == '*';
+    return op == '/' || op == '*' || op == '%';
 }
 
 /*
@@ -388,8 +392,11 @@ static node_t *get_factortail(token_stack_t *stack, node_t *left_expr, GError **
     case '/':
         op->val.op = OP_DIV;
         break;
+    case '%':
+        op->val.op = OP_MODULO;
+        break;
     default:
-        set_error(err, "Expected '*' or '/'", token);
+        set_error(err, "Expected '*', '/' or '%'", token);
         g_free(op);
         return left_expr;
     }
